@@ -125,11 +125,13 @@
 /*	bool	var_long_queue_ids;
 /*	bool	var_daemon_open_fatal;
 /*	char	*var_dsn_filter;
-/*	int	var_smtputf8_enable
+/*	int	var_smtputf8_enable;
 /*	int	var_strict_smtputf8;
 /*	char	*var_smtputf8_autoclass;
+/*	int	var_requiretls_enable;
+/*	int	var_tls_required_enable;
 /*	int     var_idna2003_compat;
-/*	int     var_compat_level;
+/*	char	*var_compatibility_level;
 /*	char	*var_drop_hdrs;
 /*	char	*var_info_log_addr_form;
 /*	bool	var_enable_orcpt;
@@ -138,6 +140,8 @@
 /*
 /*	const	char null_format_string[1];
 /*
+/*	long	compatibility_level;
+/*
 /*	int	warn_compat_break_app_dot_mydomain;
 /*	int	warn_compat_break_smtputf8_enable;
 /*	int	warn_compat_break_chroot;
@@ -150,12 +154,20 @@
 /*	int	warn_compat_break_smtpd_tls_fpt_dgst;
 /*	int	warn_compat_break_smtp_tls_fpt_dgst;
 /*	int	warn_compat_break_lmtp_tls_fpt_dgst;
+/*	int	warn_compat_relay_before_rcpt_checks;
+/*	int	warn_compat_respectful_logging;
 /*
 /*	char	*var_maillog_file;
 /*	char	*var_maillog_file_pfxs;
 /*	char	*var_maillog_file_comp;
 /*	char	*var_maillog_file_stamp;
+/*	char	*var_maillog_file_perms;
 /*	char	*var_postlog_service;
+/*
+/*	char	*var_dnssec_probe;
+/*	bool	var_relay_before_rcpt_checks;
+/*	bool	var_respectful_logging;
+/*	char	*var_known_tcp_ports;
 /* DESCRIPTION
 /*	This module (actually the associated include file) defines
 /*	the names and defaults of all mail configuration parameters.
@@ -213,10 +225,12 @@
 #include <dict.h>
 #include <dict_db.h>
 #include <dict_lmdb.h>
+#include <dict_sockmap.h>
 #include <inet_proto.h>
 #include <vstring_vstream.h>
 #include <iostuff.h>
 #include <midna_domain.h>
+#include <logwriter.h>
 
 /* Global library. */
 
@@ -227,6 +241,8 @@
 #include <verp_sender.h>
 #include <own_inet_addr.h>
 #include <mail_params.h>
+#include <compat_level.h>
+#include <config_known_tcp_ports.h>
 
  /*
   * Special configuration variables.
@@ -339,6 +355,7 @@ int     var_strict_encoding;
 int     var_verify_neg_cache;
 int     var_oldlog_compat;
 int     var_delay_max_res;
+int     var_sockmap_max_reply;
 char   *var_int_filt_classes;
 int     var_cyrus_sasl_authzid;
 
@@ -354,8 +371,10 @@ char   *var_dsn_filter;
 int     var_smtputf8_enable;
 int     var_strict_smtputf8;
 char   *var_smtputf8_autoclass;
+int	var_requiretls_enable;
+int	var_tls_required_enable;
 int     var_idna2003_compat;
-int     var_compat_level;
+char   *var_compatibility_level;
 char   *var_drop_hdrs;
 char   *var_info_log_addr_form;
 bool    var_enable_orcpt;
@@ -364,16 +383,23 @@ char   *var_maillog_file;
 char   *var_maillog_file_pfxs;
 char   *var_maillog_file_comp;
 char   *var_maillog_file_stamp;
+char   *var_maillog_file_perms;
 char   *var_postlog_service;
 
+char   *var_dnssec_probe;
+bool    var_respectful_logging;
+char   *var_known_tcp_ports;
+
 const char null_format_string[1] = "";
 
  /*
-  * Compatibility level 3.
+  * Compatibility level 3.6.
   */
 int     warn_compat_break_smtpd_tls_fpt_dgst;
 int     warn_compat_break_smtp_tls_fpt_dgst;
 int     warn_compat_break_lmtp_tls_fpt_dgst;
+int     warn_compat_relay_before_rcpt_checks;
+int     warn_compat_respectful_logging;
 
  /*
   * Compatibility level 2.
@@ -390,6 +416,11 @@ int     warn_compat_break_smtputf8_enable;
 int     warn_compat_break_chroot;
 int     warn_compat_break_relay_restrictions;
 
+ /*
+  * Parsed from var_compatibility_level;
+  */
+long    compat_level;
+
 /* check_myhostname - lookup hostname and validate */
 
 static const char *check_myhostname(void)
@@ -493,9 +524,11 @@ static void check_mail_owner(void)
      */
     if ((pwd = getpwuid(var_owner_uid)) != 0
 	&& strcmp(pwd->pw_name, var_mail_owner) != 0)
-	msg_fatal("file %s/%s: parameter %s: user %s has same user ID as %s",
+	msg_fatal("file %s/%s: parameter %s: user %s has the same"
+		  " user ID %ld as user %s",
 		  var_config_dir, MAIN_CONF_FILE,
-		  VAR_MAIL_OWNER, var_mail_owner, pwd->pw_name);
+		  VAR_MAIL_OWNER, var_mail_owner,
+		  (long) var_owner_uid, pwd->pw_name);
 }
 
 /* check_sgid_group - lookup setgid group attributes and validate */
@@ -520,9 +553,11 @@ static void check_sgid_group(void)
      */
     if ((grp = getgrgid(var_sgid_gid)) != 0
 	&& strcmp(grp->gr_name, var_sgid_group) != 0)
-	msg_fatal("file %s/%s: parameter %s: group %s has same group ID as %s",
+	msg_fatal("file %s/%s: parameter %s: group %s has the same"
+		  " group ID %ld as group %s",
 		  var_config_dir, MAIN_CONF_FILE,
-		  VAR_SGID_GROUP, var_sgid_group, grp->gr_name);
+		  VAR_SGID_GROUP, var_sgid_group,
+		  (long) var_sgid_gid, grp->gr_name);
 }
 
 /* check_overlap - disallow UID or GID sharing */
@@ -631,26 +666,26 @@ static void check_legacy_defaults(void)
 
     /*
      * Look for specific parameters whose default changed when the
-     * compatibility level changed to 3.
+     * compatibility level changed to 3.6.
      */
-    if (var_compat_level < 3) {
+    if (compat_level < compat_level_from_string(COMPAT_LEVEL_3_6, msg_panic)) {
 	if (mail_conf_lookup(VAR_SMTPD_TLS_FPT_DGST) == 0)
 	    warn_compat_break_smtpd_tls_fpt_dgst = 1;
 	if (mail_conf_lookup(VAR_SMTP_TLS_FPT_DGST) == 0)
 	    warn_compat_break_smtp_tls_fpt_dgst = 1;
 	if (mail_conf_lookup(VAR_LMTP_TLS_FPT_DGST) == 0)
 	    warn_compat_break_lmtp_tls_fpt_dgst = 1;
-    } else {
-	warn_compat_break_smtpd_tls_fpt_dgst = 0;
-	warn_compat_break_smtp_tls_fpt_dgst = 0;
-	warn_compat_break_lmtp_tls_fpt_dgst = 0;
+	if (mail_conf_lookup(VAR_RELAY_BEFORE_RCPT_CHECKS) == 0)
+	    warn_compat_relay_before_rcpt_checks = 1;
+	if (mail_conf_lookup(VAR_RESPECTFUL_LOGGING) == 0)
+	    warn_compat_respectful_logging = 1;
     }
 
     /*
      * Look for specific parameters whose default changed when the
      * compatibility level changed to 2.
      */
-    if (var_compat_level < 2) {
+    if (compat_level < compat_level_from_string(COMPAT_LEVEL_2, msg_panic)) {
 	if (mail_conf_lookup(VAR_RELAY_DOMAINS) == 0) {
 	    warn_compat_break_relay_domains = 1;
 	    if (mail_conf_lookup(VAR_FFLUSH_DOMAINS) == 0)
@@ -659,17 +694,13 @@ static void check_legacy_defaults(void)
 	if (mail_conf_lookup(VAR_MYNETWORKS) == 0
 	    && mail_conf_lookup(VAR_MYNETWORKS_STYLE) == 0)
 	    warn_compat_break_mynetworks_style = 1;
-    } else {					/* for 'postfix reload' */
-	warn_compat_break_relay_domains = 0;
-	warn_compat_break_flush_domains = 0;
-	warn_compat_break_mynetworks_style = 0;
     }
 
     /*
      * Look for specific parameters whose default changed when the
      * compatibility level changed from 0 to 1.
      */
-    if (var_compat_level < 1) {
+    if (compat_level < compat_level_from_string(COMPAT_LEVEL_1, msg_panic)) {
 	if (mail_conf_lookup(VAR_APP_DOT_MYDOMAIN) == 0)
 	    warn_compat_break_app_dot_mydomain = 1;
 
@@ -687,11 +718,6 @@ static void check_legacy_defaults(void)
 	 */
 	if (mail_conf_lookup(VAR_RELAY_CHECKS) == 0)
 	    warn_compat_break_relay_restrictions = 1;
-    } else {					/* for 'postfix reload' */
-	warn_compat_break_app_dot_mydomain = 0;
-	warn_compat_break_smtputf8_enable = 0;
-	warn_compat_break_chroot = 0;
-	warn_compat_break_relay_restrictions = 0;
     }
 }
 
@@ -699,8 +725,8 @@ static void check_legacy_defaults(void)
 
 void    mail_params_init()
 {
-    static const CONFIG_INT_TABLE first_int_defaults[] = {
-	VAR_COMPAT_LEVEL, DEF_COMPAT_LEVEL, &var_compat_level, 0, 0,
+    static const CONFIG_STR_TABLE compat_level_defaults[] = {
+	VAR_COMPAT_LEVEL, DEF_COMPAT_LEVEL, &var_compatibility_level, 0, 0,
 	0,
     };
     static const CONFIG_STR_TABLE first_str_defaults[] = {
@@ -716,7 +742,10 @@ void    mail_params_init()
 	VAR_MAILLOG_FILE_PFXS, DEF_MAILLOG_FILE_PFXS, &var_maillog_file_pfxs, 1, 0,
 	VAR_MAILLOG_FILE_COMP, DEF_MAILLOG_FILE_COMP, &var_maillog_file_comp, 1, 0,
 	VAR_MAILLOG_FILE_STAMP, DEF_MAILLOG_FILE_STAMP, &var_maillog_file_stamp, 1, 0,
+	VAR_MAILLOG_FILE_PERMS, DEF_MAILLOG_FILE_PERMS, &var_maillog_file_perms, 1, 0,
 	VAR_POSTLOG_SERVICE, DEF_POSTLOG_SERVICE, &var_postlog_service, 1, 0,
+	VAR_DNSSEC_PROBE, DEF_DNSSEC_PROBE, &var_dnssec_probe, 0, 0,
+	VAR_KNOWN_TCP_PORTS, DEF_KNOWN_TCP_PORTS, &var_known_tcp_ports, 0, 0,
 	0,
     };
     static const CONFIG_BOOL_TABLE first_bool_defaults[] = {
@@ -729,6 +758,9 @@ void    mail_params_init()
 	/* read and process the following before opening tables. */
 	VAR_SMTPUTF8_ENABLE, DEF_SMTPUTF8_ENABLE, &var_smtputf8_enable,
 	VAR_IDNA2003_COMPAT, DEF_IDNA2003_COMPAT, &var_idna2003_compat,
+	VAR_RESPECTFUL_LOGGING, DEF_RESPECTFUL_LOGGING, &var_respectful_logging,
+	VAR_REQUIRETLS_ENABLE, DEF_REQUIRETLS_ENABLE, &var_requiretls_enable,
+	VAR_TLSREQUIRED_ENABLE, DEF_TLSREQUIRED_ENABLE, &var_tls_required_enable,
 	0,
     };
     static const CONFIG_STR_FN_TABLE function_str_defaults[] = {
@@ -814,6 +846,7 @@ void    mail_params_init()
 	VAR_MIME_BOUND_LEN, DEF_MIME_BOUND_LEN, &var_mime_bound_len, 1, 0,
 	VAR_DELAY_MAX_RES, DEF_DELAY_MAX_RES, &var_delay_max_res, MIN_DELAY_MAX_RES, MAX_DELAY_MAX_RES,
 	VAR_INET_WINDOW, DEF_INET_WINDOW, &var_inet_windowsize, 0, 0,
+	VAR_SOCKMAP_MAX_REPLY, DEF_SOCKMAP_MAX_REPLY, &var_sockmap_max_reply, 1, 0,
 	0,
     };
     static const CONFIG_LONG_TABLE long_defaults[] = {
@@ -861,7 +894,10 @@ void    mail_params_init()
      * Extract compatibility level first, so that we can determine what
      * parameters of interest are left at their legacy defaults.
      */
-    get_mail_conf_int_table(first_int_defaults);
+    if (var_compatibility_level == 0)
+	compat_level_relop_register();
+    get_mail_conf_str_table(compat_level_defaults);
+    compat_level = compat_level_from_string(var_compatibility_level, msg_fatal);
     check_legacy_defaults();
 
     /*
@@ -904,6 +940,11 @@ void    mail_params_init()
 #endif
     util_utf8_enable = var_smtputf8_enable;
 
+    /*
+     * Configure the known TCP port mappings.
+     */
+    config_known_tcp_ports(VAR_KNOWN_TCP_PORTS, var_known_tcp_ports);
+
     /*
      * What protocols should we attempt to support? The result is stored in
      * the global inet_proto_table variable.
@@ -954,7 +995,11 @@ void    mail_params_init()
     check_overlap();
     dict_db_cache_size = var_db_read_buf;
     dict_lmdb_map_size = var_lmdb_map_size;
+    dict_sockmap_max_reply = var_sockmap_max_reply;
     inet_windowsize = var_inet_windowsize;
+    if (set_logwriter_create_perms(var_maillog_file_perms) < 0)
+	msg_warn("ignoring bad permissions: %s = %s",
+		 VAR_MAILLOG_FILE_PERMS, var_maillog_file_perms);
 
     /*
      * Variables whose defaults are determined at runtime, after other
